<?php

/**
 * Plugin Name: Frontend Admin Pro
 * Plugin URI: https://wordpress.org/plugins/acf-frontend-form-element/
 * Description: This awesome plugin allows you to easily display admin forms to the frontend of your site so your clients can easily edit content on their own from the frontend.
 * Version:     3.10.3

 * Update URI: https://api.freemius.com
 * Author:      Shabti Kaplan
 * Author URI:  https://www.dynamiapps.com/
 * Text Domain: acf-frontend-form-element
 * Domain Path: /languages/
 *
 */
if ( !defined( 'ABSPATH' ) ) {
    exit;
}
if ( !function_exists( 'feap_fs' ) ) {
    function feap_fs()
    {
        return false;
    }

}

if ( !class_exists( 'Front_End_Admin_Pro' ) ) {

    if ( !defined( 'FEA_VERSION' ) ) {
        define( 'FEA_VERSION', '3.10.3' );
        define( 'FEA_PATH', __FILE__ );
        define( 'FEA_NAME', plugin_basename( __FILE__ ) );
        define( 'FEA_URL', plugin_dir_url( __FILE__ ) );
        define( 'FEA_DIR', __DIR__ );
        define( 'FEA_UPDATER', __DIR__ . '/pro/updater.php' );
        define( 'FEA_TITLE', 'Frontend Admin' );
        define( 'FEA_NS', 'acf-frontend-form-element' );
        define( 'FEA_PREFIX', 'acf_frontend' );
        define( 'ALT_PREFIX', 'frontend_admin' );
        define( 'FEA_PRE', 'acf-frontend' );
    }


    if ( !function_exists( 'fea_freemius' ) ) {
        function fea_freemius()
        {
            global  $fea_freemius ;
            if ( isset( $fea_freemius ) ) {
                return $fea_freemius;
            }
            if ( !defined( 'WP_FS__PRODUCT_5212_MULTISITE' ) ) {
                define( 'WP_FS__PRODUCT_5212_MULTISITE', true );
            }
            require_once dirname( __FILE__ ) . '/includes/freemius/start.php';
            $fea_freemius = fs_dynamic_init( array(
                'id'              => '5212',
                'slug'            => FEA_NS,
                'premium_slug'    => FEA_NS . '-pro',
                'type'            => 'plugin',
                'public_key'      => 'pk_771aff8259bcf0305b376eceb7637',
                'is_premium'      => true,
                'premium_suffix'  => 'Pro',
                'has_addons' => false,
				'has_paid_plans' => false,
				'trial' => array(
				'days' => 7,
				'is_require_payment' => false,
            ),
                'has_affiliation' => false,
                'menu'            => array(
                'slug'        => 'acf-frontend-settings',
                'contact'     => false,
                'support'     => false,
                'affiliation' => false,
            ),
                'is_live'         => true,
            ) );
            return $fea_freemius;
        }

        fea_freemius();
    }

    /**
     * Main Frontend Admin Class
     *
     * The main class that initiates and runs the plugin.
     *
     * @since 1.0.0
     */
    final class Front_End_Admin_Pro
    {
        /**
         * Minimum PHP Version
         *
         * @since 1.0.0
         *
         * @var string Minimum PHP version required to run the plugin.
         */
        const  MINIMUM_PHP_VERSION = '5.2.4' ;
        /**
         * Constructor
         *
         * @since 1.0.0
         *
         * @access public
         */
        public function __construct()
        {
            global  $fea_instance ;
            if ( isset( $fea_instance ) ) {
                return;
            }
            do_action( 'front_end_admin_loaded' );
            do_action( 'front_end_admin_pro_loaded' );
            $fea_instance = $this;
            add_action( 'init', [ $this, 'i18n' ] );
            add_action( 'after_setup_theme', array( $this, 'init' ), 10 );
            require_once 'pro/features.php';
        }

        /**
         * Load Textdomain
         *
         * Load plugin localization files.
         *
         * Fired by `init` action hook.
         *
         * @since 1.0.0
         *
         * @access public
         */
        public function i18n()
        {
            load_plugin_textdomain( FEA_NS, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
        }

        /**
         * Initialize the plugin
         *
         * Load the plugin only after ACF is loaded.
         * Checks for basic plugin requirements, if one check fail don't continue,
         * If all checks have passed load the files required to run the plugin.
         *
         * Fired by `plugins_loaded` action hook.
         *
         * @since 1.0.0
         *
         * @access public
         */
        public function init()
        {

            if ( !class_exists( 'ACF' ) ) {
                add_action( 'admin_notices', [ $this, 'admin_notice_missing_acf_plugin' ] );
                return;
            }

            // Check for required PHP version

            if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
                add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
                return;
            }

            add_action( 'admin_notices', [ $this, 'admin_notice_review_plugin' ] );
            add_action( 'wp_ajax_acff-rate-plugin', array( $this, 'ajax_rate_the_plugin' ) );
            add_filter(
                'plugin_row_meta',
                [ $this, 'frontend_admin_row_meta' ],
                10,
                2
            );
            $this->plugin_includes();
        }

        public function plugin_includes()
        {
            require_once 'pro/features.php';
            require_once 'main/helpers/data-fetch.php';
            require_once 'main/helpers/addon-installer.php';
            require_once 'main/helpers/permissions.php';
            if ( did_action( 'elementor/loaded' ) ) {
                require_once __DIR__ . '/main/elementor/module.php';
            }
            require_once __DIR__ . '/main/gutenberg/module.php';
            require_once __DIR__ . '/main/frontend/module.php';
            require_once __DIR__ . '/main/admin/module.php';
        }

        public function is_license_active()
        {
            return fea_freemius()->is__premium_only();
        }

        public function set_basename()
        {
            return null;
        }

        /**
         * Admin notice
         *
         * Warning when the site doesn't have ACF installed or activated.
         *
         * @since 1.0.0
         *
         * @access public
         */
        public function admin_notice_missing_acf_plugin()
        {
            if ( isset( $_GET['activate'] ) ) {
                unset( $_GET['activate'] );
            }
            $message = sprintf(
                /* translators: 1: Plugin name 2: Advanced Custom Fields */
                esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', FEA_NS ),
                '<strong>' . esc_html__( FEA_TITLE, FEA_NS ) . '</strong>',
                '<strong>' . esc_html__( 'Advanced Custom Fields', FEA_NS ) . '</strong>'
            );
            printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
        }

        public function ajax_upgrade_pro_dismiss()
        {
            check_admin_referer( 'acff_dismiss_pro_nonce', '_n' );
            update_option( 'acff_pro_trial_dismiss', true );
            echo  1 ;
            exit;
        }

        public function admin_notice_review_plugin()
        {
            if ( !is_admin() ) {
                return;
            }
            $min_submits = get_option( 'acff_min_submits_trigger', 10 );
            $submits_count = get_option( 'frontend_admin_submissions_all_time', 0 );
            if ( $min_submits == -1 || $submits_count < $min_submits ) {
                return;
            }
            $image_path = FEA_URL . 'assets/icon.png';
            $review_url = 'https://wordpress.org/support/view/plugin-reviews/' . FEA_NS . '?rate=5#postform';
            ?>
            <div class="notice notice-info acff-rate-action" style="padding-right: 48px">
            <?php
            printf( __( "Hey, I noticed you've received over %d submissions on %s already - that's awesome! I am so glad you are enjoying my plugin! Please take a minute to help our business grow by leaving a review.", FEA_NS ), $min_submits, FEA_TITLE );
            ?>
            <strong><em>~ Shabti Kaplan</em></strong>
            <ul data-nonce="<?php
            echo  wp_create_nonce( 'acff_rate_action_nonce' ) ;
            ?>">
            <li><a data-rate-action="do-rate" href="#" data-href="<?php
            echo  esc_url( $review_url ) ;
            ?>"><?php
            esc_html_e( 'Ok, you deserve it', FEA_NS );
            ?></a></li>
            <li><a data-rate-action="done-rating" href="#"><?php
            esc_html_e( 'I already did', FEA_NS );
            ?></a></li>
            <li><a data-rate-action="not-yet" href="#"><?php
            esc_html_e( 'Nope, maybe later', FEA_NS );
            ?></a></li>
            </ul>
            <div><img width="40px" src="<?php
            echo  esc_url( $image_path ) ;
            ?>" style="width:40px;margin:10px"/></div>

                    </div>
            <?php
            $min = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '-min' );
            wp_enqueue_script(
                'fea-rate-plugin',
                FEA_URL . 'assets/js/rate-plugin' . $min . '.js',
                array( 'acf' ),
                FEA_VERSION,
                true
            );
            wp_localize_script( 'fea-rate-plugin', 'fa', array(
                'ajaxurl' => admin_url( 'admin-ajax.php' ),
            ) );
        }

        public function ajax_rate_the_plugin()
        {
            // Continue only if the nonce is correct
            check_admin_referer( 'acff_rate_action_nonce', '_n' );
            $min_submits = get_option( 'acff_min_submits_trigger', 10 );
            if ( -1 === $min_submits ) {
                exit;
            }
            $rate_action = wp_kses( $_POST['rate_action'], 'strip' );

            if ( 'do-rate' === $rate_action ) {
                $min_submits = -1;
            } else {

                if ( $min_submits === 10 ) {
                    $min_submits = 100;
                } else {

                    if ( $min_submits === 100 ) {
                        $min_submits = 1000;
                    } else {
                        $min_submits = -1;
                    }

                }

            }

            update_option( 'acff_min_submits_trigger', $min_submits );
            echo  1 ;
            exit;
        }

        /**
         * Admin notice
         *
         * Warning when the site doesn't have a minimum required PHP version.
         *
         * @since 1.0.0
         *
         * @access public
         */
        public function admin_notice_minimum_php_version()
        {
            if ( isset( $_GET['activate'] ) ) {
                unset( $_GET['activate'] );
            }
            $message = sprintf(
                /* translators: 1: Plugin name 2: PHP 3: Required PHP version */
                esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', FEA_NS ),
                '<strong>' . esc_html__( FEA_TITLE, FEA_NS ) . '</strong>',
                '<strong>' . esc_html__( 'PHP', FEA_NS ) . '</strong>',
                self::MINIMUM_PHP_VERSION
            );
            printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
        }

        public function frontend_admin_row_meta( $links, $file )
        {

            if ( FEA_NAME == $file ) {
                $row_meta = array(
                    'video' => '<a href="' . esc_url( 'https://www.youtube.com/channel/UC8ykyD--K6pJmGmFcYsaD-w/playlists' ) . '" target="_blank" aria-label="' . esc_attr__( 'Video Tutorials', FEA_NS ) . '" >' . esc_html__( 'Video Tutorials', FEA_NS ) . '</a>',
                );
                return array_merge( $links, $row_meta );
            }

            return (array) $links;
        }

    }
    new Front_End_Admin_Pro();
}
