<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}


if( ! function_exists( 'fea_instance' ) ){
	function fea_instance()
	{
		global $fea_instance;
		
		if ( !isset( $fea_instance ) ) {      
			$fea_instance = new stdClass();
		}
		
		return $fea_instance;
	}
}

function feadmin_form_types(){
	$form_types = array( 
		'general' => __( 'Frontend Form', FEA_NS ),
		__( 'Post', FEA_NS ) => array(
			'new_post' => __( 'New Post Form', FEA_NS ),
			'edit_post'	=> __( 'Edit Post Form', FEA_NS ),
			'duplicate_post' => __( 'Duplicate Post Form', FEA_NS ),
			'delete_post' => __( 'Delete Post Button', FEA_NS ),
			'status_post' => __( 'Post Status Button', FEA_NS ),
		),
		__( 'User', FEA_NS ) => array(
			'new_user'	=> __( 'New User Form', FEA_NS ),
			'edit_user'	=> __( 'Edit User Form', FEA_NS ),
			'delete_user' => __( 'Delete User Button', FEA_NS ),
		),
		__( 'Term', FEA_NS ) => array(
			'new_term'	=> __( 'New Term Form', FEA_NS ),
			'edit_term'	=> __( 'Edit Term Form', FEA_NS ),
			'delete_term' => __( 'Delete Term Button', FEA_NS ),
		),
	);
	if( fea_instance()->is_license_active() ){
		if ( class_exists( 'woocommerce' ) ){
			$form_types = array_merge( $form_types, array( 
				__( 'Product', FEA_NS ) => array(
					'new_product'	=> __( 'New Product Form', FEA_NS ),
					'edit_product'	=> __( 'Edit Product Form', FEA_NS ),
					'duplicate_product' => __( 'Duplicate Product Form', FEA_NS ),
					'delete_product' => __( 'Delete Product Button', FEA_NS ),
					'status_product' => __( 'Post Status Button', FEA_NS ),
				),
			) );
		}
		$form_types['edit_options'] = __( 'Edit Options Form', FEA_NS );
	}
	return $form_types;
}

function feadmin_user_exists( $id ){

    global $wpdb;

    $count = $wpdb->get_var($wpdb->prepare( "SELECT COUNT(*) FROM $wpdb->users WHERE ID = %d", $id ) );

    if( $count == 1 ) return true;
	
	return false;

}

function feadmin_get_field_data( $type = null, $form_fields = false ){
	$field_types = [];
	if( ! $form_fields ){
		$GLOBALS['only_acf_field_groups'] = 1;
	}
	$acf_field_groups = acf_get_field_groups();
	$GLOBALS['only_acf_field_groups'] = 0;
	// bail early if no field groups
	if( empty($acf_field_groups) ) die();
	// loop through array and add to field 'choices'
	if( $acf_field_groups ) {   
		foreach( $acf_field_groups as $field_group ) {
			if( ! empty( $field_group['frontend_admin_group'] ) ) continue;
			$field_group_fields = acf_get_fields( $field_group['key'] );
			if( is_array( $field_group_fields ) ) { 
				foreach( $field_group_fields as $acf_field ) {										
					if( $type ){
						if( ( is_array( $type ) && in_array( $acf_field['type'], $type ) ) || ( ! is_array( $type ) && $acf_field['type'] == $type ) ){
							$field_types[$acf_field['key']] = $acf_field['label']; 
						}
					}else{
						$field_types[$acf_field['key']]['type'] = $acf_field['type']; 
						$field_types[$acf_field['key']]['label'] = $acf_field['label'];  
						$field_types[$acf_field['key']]['name'] = $acf_field['name'];  
					}
				}
			} 
		}
	}
	return $field_types;
}	

function feadmin_user_id_fields(){
	$fields = feadmin_get_acf_field_choices( array( 'type' => 'user' ) );
	$keys = array_merge( ['[author]' => __( 'Post Author', FEA_NS ) ],  $fields );
	return $keys;
}

function feadmin_get_user_roles( $exceptions = [], $all = false ){
	if( ! current_user_can('administrator') ) $exceptions[] = 'administrator';
	
	$user_roles = array();

	if( $all ){
		$user_roles['all'] = __( 'All', FEA_NS );
	}
	global $wp_roles;
	// loop through array and add to field 'choices'
		foreach( $wp_roles->roles as $role => $settings ) {
			if( ! in_array( strtolower( $role ), $exceptions ) ){
				$user_roles[ $role ] = $settings['name']; 
			}
		}
	return $user_roles;
}
function feadmin_get_user_caps( $exceptions = [], $all = false ){
	$user_caps = array();

	$data = get_userdata( get_current_user_id() );
 
	if ( is_object( $data) ) {
    	$current_user_caps = $data->allcaps;
		foreach( $current_user_caps as $cap => $true ) {
			if( ! in_array( strtolower( $cap ), $exceptions ) ){
				$user_caps[ $cap ] = $cap; 
			}
		}
	}

	return $user_caps;
}

function feadmin_get_acf_group_choices(){
	$field_group_choices = [];
	$acf_field_groups = acf_get_field_groups();
	// loop through array and add to field 'choices'
	if( is_array( $acf_field_groups ) ) {        
		foreach( $acf_field_groups as $field_group ) {
			if( is_array( $field_group ) && ! isset( $field_group['frontend_admin_group'] ) ){
				$field_group_choices[ $field_group['key'] ] = $field_group['title']; 
			}
		}
	}
	return $field_group_choices;
}	

/* add_filter('acf/get_fields', function( $fields, $parent ){
	$group = explode( 'acfef_', $parent['key'] ); 

	if( empty( $group[1] ) ) return $fields;

	return array();
}, 5, 2);
 */

function feadmin_get_acf_field_choices( $filter = array(), $return = 'label' ){
	$all_fields = []; 
	if( isset( $filter['groups'] ) ){
		$acf_field_groups = $filter['groups'];
	}else{
		$acf_field_groups = acf_get_field_groups( $filter );
	}

	// bail early if no field groups
	if( empty($acf_field_groups) ) return array();

	foreach( $acf_field_groups as $group ) {
		if( ! is_array( $group ) ) $group = acf_get_field_group( $group );
		if( ! empty( $field_group['frontend_admin_group'] ) ) continue;

		$group_fields = acf_get_fields( $group );

		if( is_array( $group_fields ) ) { 
			foreach( $group_fields as $acf_field ) {
				if( ! is_array( $acf_field ) ) continue;

				$acf_field_key = $acf_field['type'] == 'clone' ? $acf_field['__key'] : $acf_field['key'];
				if( ! empty( $filter['type'] ) && $filter['type'] == $acf_field['type'] ){
					$all_fields[ $acf_field['name'] ] = $acf_field[$return];
				}else{
					if( isset( $filter['groups'] ) ){
						$all_fields[$acf_field_key] = $acf_field[$return]; 
					}else{
						$all_fields[$acf_field_key] = $acf_field[$return]; 
					}
				}
			}
		} 
	}

	return $all_fields;	
}	

function feadmin_get_post_type_choices(){
	$post_type_choices = [];
	$args = array();
	$output = 'names'; // names or objects, note names is the default
	$operator = 'and'; // 'and' or 'or'
	$post_types = get_post_types( $args, $output, $operator ); 
	// loop through array and add to field 'choices'
	if( is_array( $post_types ) ) {        
		foreach( $post_types as $post_type ) {
			$post_type_choices[ $post_type ] = str_replace( '_', ' ', ucfirst( $post_type ) ); 
		}
	}
	return $post_type_choices;
}

function feadmin_get_random_string($length = 15) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';
    for ($i = 0; $i < $length; $i++) {
        $randomString .= $characters[rand(0, $charactersLength - 1)];
    }
    return $randomString;
}


function feadmin_get_client_ip() {
	$server_ip_keys = [
		'HTTP_CLIENT_IP',
		'HTTP_X_FORWARDED_FOR',
		'HTTP_X_FORWARDED',
		'HTTP_X_CLUSTER_CLIENT_IP',
		'HTTP_FORWARDED_FOR',
		'HTTP_FORWARDED',
		'REMOTE_ADDR',
	];

	foreach ( $server_ip_keys as $key ) {
		if ( isset( $_SERVER[ $key ] ) && filter_var( $_SERVER[ $key ], FILTER_VALIDATE_IP ) ) {
			return $_SERVER[ $key ];
		}
	}

	// Fallback local ip.
	return '127.0.0.1';
}

function feadmin_get_site_domain() {
	return str_ireplace( 'www.', '', parse_url( home_url(), PHP_URL_HOST ) );
}


function feadmin_get_esc_attrs( $attrs ) {
	$html = '';
	
	// Loop over attrs and validate data types.
	foreach( $attrs as $k => $v ) {
		
		// String (but don't trim value).
		if( is_string($v) && ($k !== 'value') ) {
			$v = trim($v);
			
		// Boolean	
		} elseif( is_bool($v) ) {
			$v = $v ? 1 : 0;
			
		// Object
		} elseif( is_array($v) || is_object($v) ) {
			$v = json_encode($v);
		}
		
		// Generate HTML.
		$html .= sprintf( ' %s="%s"', esc_attr($k), esc_attr($v) );
	}
	
	// Return trimmed.
	return trim( $html );
}
	
function feadmin_duplicate_slug( $prefix = '' ) {	
	static $i;
	if ( null === $i ) {
		$i = 2;
	} else {
		$i ++;
	}
	$new_slug = sprintf( '%s_copy%s', $prefix, $i );
	if ( ! feadmin_slug_exists( $new_slug ) ) {
		return $new_slug;
	} else {
		return feadmin_duplicate_slug( $prefix );
	}
}

function feadmin_slug_exists($post_name) {
    global $wpdb;
    if($wpdb->get_row("SELECT post_name FROM $wpdb->posts WHERE post_name = '$post_name'", 'ARRAY_A')) {
        return true;
    } else {
        return false;
    }
}

function feadmin_parse_args( $args, $defaults ) {
	$new_args = (array) $defaults;

	if( ! is_array( $args ) ) return $defaults;                 
	foreach ( $args as $key => $value ) {
		if ( is_array( $value ) && isset( $new_args[ $key ] ) ) {
			$new_args[ $key ] = feadmin_parse_args( $value, $new_args[ $key ] );
		}else {
			$new_args[ $key ] = $value;
		}
	}

	return $new_args;
}

function feadmin_edit_mode(){
	$edit_mode = false;

	if( ! empty( fea_instance()->elementor ) ) $edit_mode = \Elementor\Plugin::$instance->editor->is_edit_mode();	

	if( ! empty( $GLOBALS['admin_form']['preview_mode'] ) ) $edit_mode = true;

	return $edit_mode;
}

function feadmin_get_product_object(){
	if( isset( $GLOBALS['admin_form']['save_to_product'] ) ){
		$form = $GLOBALS['admin_form'];

		if( $form['save_to_product'] == 'edit_product' ){				
			return wc_get_product( $form['product_id'] );	
		}
	}
	return false;
}

function feadmin_get_field_type_groups( $type = 'all' )
{
	$fields = [];
	if( $type == 'all' ){
		$fields['acf'] = array(
			'label'   => __( 'ACF Field', FEA_NS ),
			'options' => array(
				'ACF_fields'       => __( 'ACF Fields', FEA_NS ),
				'ACF_field_groups' => __( 'ACF Field Groups', FEA_NS ),
			),
		);
		$fields['layout'] = array(
			'label'   => __( 'Layout', FEA_NS ),
			'options' => array(
				'message' => __( 'Message', FEA_NS ),
				'column'  => __( 'Column', FEA_NS ),
				//'tab'  => __( 'Tab', FEA_NS ),
			),
		);
	}
	if( $type == 'all' || $type == 'post' ){
		$fields['post'] = array(
			'label'   => __( 'Post' ),
			'options' => array(
			'title'          => __( 'Post Title', FEA_NS ),
			'slug'           => __( 'Slug', FEA_NS ),
			'content'        => __( 'Post Content', FEA_NS ),
			'featured_image' => __( 'Featured Image', FEA_NS ),
			'excerpt'        => __( 'Post Excerpt', FEA_NS ),
			'categories'     => __( 'Categories', FEA_NS ),
			'tags'           => __( 'Tags', FEA_NS ),
			'author'         => __( 'Post Author', FEA_NS ),
			'published_on'   => __( 'Published On', FEA_NS ),
			'post_type'      => __( 'Post Type', FEA_NS ),
			'menu_order'     => __( 'Menu Order', FEA_NS ),
			'allow_comments' => __( 'Allow Comments', FEA_NS ),
			'taxonomy'       => __( 'Custom Taxonomy', FEA_NS ),
		),
		);
	}
	if( $type == 'all' || $type == 'user' ){
		$fields['user'] = array(
			'label'   => __( 'User', FEA_NS ),
			'options' => array(
			'username'         => __( 'Username', FEA_NS ),
			'password'         => __( 'Password', FEA_NS ),
			'confirm_password' => __( 'Confirm Password', FEA_NS ),
			'email'            => __( 'Email', FEA_NS ),
			'first_name'       => __( 'First Name', FEA_NS ),
			'last_name'        => __( 'Last Name', FEA_NS ),
			'nickname'         => __( 'Nickname', FEA_NS ),
			'display_name'     => __( 'Display Name', FEA_NS ),
			'bio'              => __( 'Biography', FEA_NS ),
			'role'             => __( 'Role', FEA_NS ),
		),
		);
	}
	if( $type == 'all' || $type == 'term' ){

		$fields['term'] = array(
			'label'   => __( 'Term', FEA_NS ),
			'options' => array(
				'term_name' => __( 'Term Name', FEA_NS ),
				'term_slug' => __( 'Term Slug', FEA_NS ),
				'term_description' => __( 'Term Description', FEA_NS ),
			),
		);
	}

	if ( fea_instance()->is_license_active() ) {
		if( $type == 'all' || $type == 'options' ){
			$fields['options']  = array(
				'label' => __( 'Site', FEA_NS ),
				'options' => array(
					'site_title' => __( 'Site Title', FEA_NS ),
					'site_tagline' => __( 'Site Tagline', FEA_NS ),
					'site_logo' => __( 'Site Logo', FEA_NS ),
				)
			);
		}
		/* if( $type == 'all' || $type == 'comment' ){
			$fields['comment']  = array(
				'label' => __( 'Comment', FEA_NS ),
				'options' => array(
					'comment' => __( 'Comment Body', FEA_NS ),
					'author' => __( 'Comment Author', FEA_NS ),
					'author_email' => __( 'Comment Author Email', FEA_NS ),
				)
			);
		} */
		if( $type == 'all' ){
			$fields['security'] = array(
				'label' => __( 'Security', FEA_NS ),
				'options' => array(
					'recaptcha' => __( 'Recaptcha', FEA_NS ),
				)
			);
		}
		if ( class_exists( 'woocommerce' ) ){	
			if( $type == 'all' || $type == 'product' ){
				$fields['product_type'] = array(
					'label' => __( 'Product Type', FEA_NS ),
					'options' => array(
						'product_type' => __( 'Product Types', 'woocommerce' ),
						'is_virtual' => __( 'Virtual', 'woocommerce' ),
						'is_downloadable' => __( 'Downloadable', 'woocommerce' ),
					)
				);	
				$fields['product']  = array(
					'label' => __( 'Product Information', FEA_NS ),
					'options' => array(
						'product_title' => __( 'Product Title', FEA_NS ),
						'product_slug' => __( 'Slug', FEA_NS ),
						'price' => __( 'Price', FEA_NS ),
						'sale_price' => __( 'Sale Price', FEA_NS ),
						'description' => __( 'Description', FEA_NS ),
						'main_image' => __( 'Main Image', FEA_NS ),
						'images' => __( 'Product Images', FEA_NS ),
						'short_description' => __( 'Short Description', FEA_NS ),
						'product_categories' => __( 'Categories', FEA_NS ),
						'product_tags' => __( 'Tags', FEA_NS ),
						'tax_status' => __( 'Tax Status', FEA_NS ),
						'tax_class' => __( 'Tax Class', FEA_NS ),
					)
				);		
				$fields['product_downloadable'] = array(
					'label' => __( 'Product Downloads', FEA_NS ),
					'options' => array(
						'download_limit' => __( 'Download Limit', 'woocommerce' ),
						'download_expiry' => __( 'Download Expiry', 'woocommerce' ),
						'downloadable_files' => __( 'Downloadable Files', 'woocommerce' ),
					)
				);
				$fields['product_shipping'] = array(
					'label' => __( 'Product Shipping', FEA_NS ),
					'options' => array(
						'product_weight' => __( 'Weight', 'woocommerce' ),
						'product_length' => __( 'Length', 'woocommerce' ),
						'product_width' => __( 'Width', 'woocommerce' ),
						'product_height' => __( 'Height', 'woocommerce' ),
						'product_shipping_class' => __( 'Shipping Class', 'woocommerce' ),
					)
				);
				$fields['product_external'] = array(
					'label' => __( 'External/Affiliate product', 'woocommerce' ),
					'options' => array(
						'external_url' => __( 'Product URL', FEA_NS ),
						'button_text' => __( 'Button Text', FEA_NS ),
					)
				);
				$fields['product_linked'] = array(
					'label' => __( 'Linked Products', FEA_NS ),
					'options' => array(
						'grouped_products' => __( 'Grouped Products', FEA_NS ),
						'upsells' => __( 'Upsells', FEA_NS ),
						'cross_sells' => __( 'Cross Sells', FEA_NS ),
					)
				);
				$fields['product_attributes'] = array(
					'label' => __( 'Product Attributes', FEA_NS ),
					'options' => array(
						'attributes' => __( 'Attributes', FEA_NS ),
						'variations' => __( 'Variations', FEA_NS ),
					)
				);
				$fields['product_inventory'] = array(
					'label' => __( 'Product Inventory', FEA_NS ),
					'options' => array(
						'sku' =>  __( 'sku', FEA_NS ),
						'stock_status' =>  __( 'Stock Status', FEA_NS ),
						'sold_individually' =>  __( 'Sold Individually', FEA_NS ), 
						'manage_stock' =>  __( 'Manage Stock', FEA_NS ), 
						'stock_quantity' =>  __( 'Stock Quantity', FEA_NS ), 
						'allow_backorders' =>  __( 'Allow Backorders', FEA_NS ), 
						'low_stock_threshold' =>  __( 'Low Stock Threshold', FEA_NS ), 
					)
				);
				$fields['product_advanced'] = array(
					'label' => __( 'Advanced Product Options', FEA_NS ),
					'options' => array(
						'product_purchase_note' =>  __( 'Purchase Note', FEA_NS ),
						'product_menu_order' =>  __( 'Menu Order', FEA_NS ),
						'product_enable_reviews' =>  __( 'Enable Reviews', FEA_NS ),
					)
				);
			}
		}
		if( $type == 'all' ){
			$fields['layout']['options']['step'] = __( 'Step', FEA_NS );
		}
	}
	return $fields;
}


/*
*  get_selected_field
*
*  This function will return the label for a given clone choice
*
*  @type    function
*  @date    17/06/2016
*  @since   5.3.8
*
*  @param   $selector (mixed)
*  @return  (string)
*/

function feadmin_get_selected_field( $selector = '', $type = '' ) {

	// bail early no selector
	if ( ! $selector ) {
		return '';
	}


	if ( is_numeric( $selector ) ) {
		return get_post_field( 'post_title', $selector );
	}	
	// ajax_fields
	if ( isset( $_POST['fields'][ $selector ] ) ) {

		return feadmin_field_choice( $_POST['fields'][ $selector ] );

	}

	// field
	if ( acf_is_field_key( $selector ) ) {

		return feadmin_field_choice( acf_get_field( $selector ) );

	}

	// group
	if ( acf_is_field_group_key( $selector ) ) {

		return feadmin_group_choice( acf_get_field_group( $selector ) );

	}
	if ( feadmin_is_admin_form_key( $selector ) ) {

		return feadmin_group_choice( fea_instance()->form_display->get_form( $selector ) );

	}

	// return
	return $selector;

}
/*
*  feadmin_field_choice
*
*  This function will return the text for a field choice
*
*  @type    function
*  @date    20/07/2016
*  @since   5.4.0
*
*  @param   $field (array)
*  @return  (string)
*/

function feadmin_field_choice( $field ) {

	// bail early if no field
	if ( ! $field ) {
		return __( 'Unknown field', 'acf' );
	}

	// title
	$title = $field['label'] ? $field['label'] : __( '(no title)', 'acf' );

	// append type
	$title .= ' (' . $field['type'] . ')';

	// ancestors
	// - allow for AJAX to send through ancestors count
	$ancestors = isset( $field['ancestors'] ) ? $field['ancestors'] : count( acf_get_field_ancestors( $field ) );
	$title     = str_repeat( '- ', $ancestors ) . $title;

	// return
	return $title;

}


/*
*  feadmin_group_choice
*
*  This function will return the text for a group choice
*
*  @type    function
*  @date    20/07/2016
*  @since   5.4.0
*
*  @param   $field_group (array)
*  @return  (string)
*/

function feadmin_group_choice( $field_group ) {

	// bail early if no field group
	if ( ! $field_group ) {
		return __( 'Unknown field group', 'acf' );
	}

	// return
	return sprintf( __( 'All fields from %s', FEA_NS ), $field_group['title'] );

}

/*
*  get_selected_fields
*
*  This function will return an array of choices data for Select2
*
*  @type    function
*  @date    17/06/2016
*  @since   5.3.8
*
*  @param   $value (mixed)
*  @return  (array)
*/

function feadmin_get_selected_fields( $value, $choices = array() ) {
	// bail early if no $value
	if ( empty( $value ) ) {
		return $choices;
	}

	// force value to array
	$value = acf_get_array( $value );

	// loop
	foreach ( $value as $v ) {

		$choices[ $v ] = feadmin_get_selected_field( $v );

	}

	// return
	return $choices;

}
function feadmin_is_admin_form_key( $id ){
	if ( is_string( $id ) && substr( $id, 0, 5 ) === 'form_' ){
		return true;
	}
	return false;
}

function feadmin_form_choices( $choices = array() ){
	$args = array(
		'post_type' => 'admin_form',
		'posts_per_page' => '-1',
		'post_status' => 'any',
	);
	
	$forms = get_posts( $args );	

	if( empty( $forms ) ) return $choices;
	
	foreach( $forms as $form ){
		$choices[$form->ID] = $form->post_title;
	}

	return $choices;
}

/*
* fea_encrypt
*
*  This function will encrypt an array using PHP
*  https://bhoover.com/using-php-openssl_encrypt-openssl_decrypt-encrypt-decrypt-data/
*
*  @type    function
*  @date    02/11/22
*  @since   3.9.11
*
*  @param   $data (array)
*  @return  (string)
*/


function fea_encrypt( $data = [] ) {
	if( empty( $data ) ) return false;

	$data = json_encode( $data );

	// bail early if no encrypt function
	if ( ! function_exists( 'openssl_encrypt' ) ) {
		return base64_encode( $data );
	}

	// generate a key
	$key = wp_hash( 'fea_encrypt' );

	// Generate an initialization vector
	$iv = openssl_random_pseudo_bytes( openssl_cipher_iv_length( 'aes-256-cbc' ) );

	// Encrypt the data using AES 256 encryption in CBC mode using our encryption key and initialization vector.
	$encrypted_data = openssl_encrypt( $data, 'aes-256-cbc', $key, 0, $iv );

	// The $iv is just as important as the key for decrypting, so save it with our encrypted data using a unique separator (::)
	return base64_encode( $encrypted_data . '::' . $iv );

}

/*
*  fea_decrypt
*
*  This function will decrypt an encrypted string using PHP
*  https://bhoover.com/using-php-openssl_encrypt-openssl_decrypt-encrypt-decrypt-data/
*
*  @type    function
*  @date    02/11/22
*  @since   3.9.11
*
*  @param   $data (string)
*  @return  (array)
*/

function fea_decrypt( $data = '' ) {
	if( empty( $data ) ) return false;

	// bail early if no decrypt function
	if ( ! function_exists( 'openssl_decrypt' ) ) {
		return json_decode( base64_decode( $data ), true );
	}

	// generate a key
	$key = wp_hash( 'fea_encrypt' );

	// To decrypt, split the encrypted data from our IV - our unique separator used was "::"
	list($encrypted_data, $iv) = explode( '::', base64_decode( $data ), 2 );

	// decrypt
	$data = openssl_decrypt( $encrypted_data, 'aes-256-cbc', $key, 0, $iv );

	return json_decode( $data, true );

}