<?php
namespace Frontend_Admin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
if( ! class_exists( 'Pro_Features' ) ) :

	class Pro_Features{

        public function permissions_settings( $fields ){
            if( empty( $fields[2]['sub_fields'] ) ) return $fields;
            $fields[2]['sub_fields'] = array_merge( $fields[2]['sub_fields'], array(
                array (
                    'key' => 'allowed_submits',
                    'label' => __( 'Allowed Submissions', FEA_NS ),
                    'type' => 'number',
                    'instructions' => __( 'Limit the amount of times this form can be submitted', FEA_NS ),
                    'conditional_logic' => 0,
                    'min' => 1,
                    'wrapper' => array (
                        'width' => '30',
                        'class' => '',
                        'id' => '',
                    ),
                    'conditional_logic' => array(
                        array(
                            array(
                                'field' => 'who_can_see',
                                'operator' => '==',
                                'value' => 'logged_in',
                            ),
                        ),
                    ),
                ),
                array(
                    'key' => 'limit_reached',
                    'label' => __( 'No Permissions Message', FEA_NS ),
                    'type' => 'select',
                    'instructions' => '',
                    'required' => 0,
                    'choices' => array(
                        'show_nothing'   => __( 'None', FEA_NS ),
                        'show_message'   => __( 'Message', FEA_NS ),
                        'custom_content' => __( 'Custom Content', FEA_NS ),
                    ),
                    'conditional_logic' => array(
                        array(
                            array(
                                'field' => 'allowed_submits',
                                'operator' => '!=empty',
                            ),
                            array(
                                'field' => 'who_can_see',
                                'operator' => '==',
                                'value' => 'logged_in',
                            ),
                        ),
                    ),
                ),	
                array(
                    'key' => 'limit_reached_message',
                    'label' => '',
                    'type' => 'textarea',
                    'instructions' => '',
                    'required' => 0,
                    'rows' => 3,
                    'placeholder' => __( 'You have submitted this form the maximum amount of times allowed', FEA_NS ),
                    'default_value' => __( 'You have submitted this form the maximum amount of times allowed', FEA_NS ),
                    'conditional_logic' => array(
                        array(
                            array(
                                'field' => 'limit_reached',
                                'operator' => '==',
                                'value' => 'show_message',
                            ),
                            array(
                                'field' => 'allowed_submits',
                                'operator' => '!=empty',
                            ),
                            array(
                                'field' => 'who_can_see',
                                'operator' => '==',
                                'value' => 'logged_in',
                            ),
                        ),
                        
                    ),
                ),	
                array(
                    'key' => 'limit_reached_content',
                    'label' => '',
                    'type' => 'wysiwyg',
                    'instructions' => '',
                    'required' => 0,
                    'conditional_logic' => array(
                        array(
                            array(
                                'field' => 'limit_reached',
                                'operator' => '==',
                                'value' => 'custom_content',
                            ),
                            array(
                                'field' => 'allowed_submits',
                                'operator' => '!=empty',
                            ),
                            array(
                                'field' => 'who_can_see',
                                'operator' => '==',
                                'value' => 'logged_in',
                            ),
                        ),
                    ),
                ),	
            ) );

            return $fields;
        } 

        public function scripts(){
			$min = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '-min';
	
            wp_register_style( 'fea-public-pro', FEA_URL . 'pro/assets/css/frontend-admin-pro'.$min.'.css', array(), FEA_VERSION );
            wp_register_script( 'fea-public-pro', FEA_URL . 'pro/assets/js/frontend-admin-pro'.$min.'.js', array( 'fea-public' ), FEA_VERSION, true );
			
		}

        public function pro_tabs( $tabs ){
			if ( class_exists( 'woocommerce' ) ){
				$tabs['product'] = __( 'Product', FEA_NS );
			}
            $tabs['steps'] = __( 'Multi Step', FEA_NS );
            return $tabs;
        }

        
        public function __construct() {
            global $fea_instance;
            if ( ! $fea_instance->is_license_active() ) return;

            add_action( 'wp_loaded', array( $this, 'scripts' ) );

            add_filter( 'frontend_admin/forms/settings_tabs', [ $this, 'pro_tabs' ] );
            add_filter( 'frontend_admin/forms/settings/permissions', [ $this, 'permissions_settings' ] );

            
        }
	}

    new Pro_Features;
endif;	