<?php
namespace Frontend_Admin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
if( ! class_exists( 'Pro_License' ) ) :

	class Pro_License{

        private $api_url = '';

        function pro_tabs( $tabs ){
            global $fea_instance;
			//$this->tabs['pdf'] = 'PDF Generator';
			$tabs['payments'] = 'Payments';
			$tabs['license'] = 'License';

            return $tabs; 
        }

	
        public function get_settings_fields( $field_keys ){
            $license = get_option( 'fea_main_license_key' );
            $is_valid = get_option( 'fea_main_license_valid' );
            $instruction = '';
    
            if( $is_valid ){
                $instruction = sprintf( __( 'License activated successfully', FEA_NS ), $is_valid );
            }else{
                if( $license ){
                    $instruction = __( 'Your license is not valid', FEA_NS );
                }else{
                    $instruction = __( 'Enter your license key to activate pro features', FEA_NS );
                }
            }
    
            $local_fields = array(
                'fea_main_license_key' => array(
                    'label' => __( 'License Key', FEA_NS ),
                    'type' => 'text',
                    'instructions' => $instruction,
                    'required' => 0,
                    'default_value' => $license,
                    'wrapper' => array(
                        'width' => '50.1',
                        'class' => '',
                        'id' => '',
                    ),
                ),
            );
            
            return $local_fields;
        } 
    
        /**
         * Check license key when user submits it.
         *
         *
         * @param array $_transient_data Update array build by WordPress.
         *
         * @return array Modified update array with custom plugin data.
         * @uses api_request()
         */
        public function check_license( $form ) {
            if( $form['hidden_fields']['admin_page'] == 'license' ){ 
                $key = $form['record']['fields']['admin_options']['fea_main_license_key']['value'];

                $is_valid = 0;

                if( $key ){
                    $api_params = array(
                        'action'       => 'activate',
                        'license_key'  => $key,
                        'license_url'  => home_url(),
                    );
                    $request = wp_remote_post(
                        $this->api_url,
                        array(
                            'timeout'   => 15,
                            'sslverify' => $this->verify_ssl(),
                            'body'      => $api_params,
                        )
                    );

                    if ( ! is_wp_error( $request ) ) {
                        $request = json_decode( wp_remote_retrieve_body( $request ) );

                        if( isset( $request->license_status ) && $request->license_status == 'valid' ){
                            $is_valid = 1;
                        }
                    }
                }

                update_option( 'fea_main_license_valid', $is_valid );

            }

        }
    
    	/**
         * Returns if the SSL of the store should be verified.
         *
         * @return bool
         * @since  1.0
         */
        private function verify_ssl() {
            return (bool) apply_filters( 'paddlepress_api_request_verify_ssl', true, $this );
        }
        public function __construct( $_api_url ) {
            $this->api_url = $_api_url;

			add_filter( FEA_PREFIX.'/admin_tabs', array( $this, 'pro_tabs' ), 15 );
            add_filter( FEA_PREFIX.'/license_fields', [ $this, 'get_settings_fields'] );
            add_action( 'frontend_admin/save_admin_options', array( $this, 'check_license' ), 10, 1 );
    
        }
	}


endif;	